#include <avr/io.h>
#include <Arduino.h>
#include "TouchBtn.h"
TouchBtn::TouchBtn(volatile uint8_t *tx_port,volatile uint8_t *tx_dir,uint8_t tx_pin,uint8_t rx_channel){
    _tx_port = tx_port;
    _tx_dir = tx_dir;
    _tx_pin = tx_pin;
    _channel = rx_channel;
    thershold = 0;
    filt = 0.0;
    eps = 0.005;
    _state = false;
    _has_changed = false;
}
void TouchBtn::begin(int touch_threshold, uint8_t debounce_ms = 50){
    _debounce_ms = debounce_ms;
    _touch_threshold = touch_threshold;
    set(*_tx_dir,_tx_pin);
}
void TouchBtn::calibrate(){
    uint16_t value = read();
    if(thershold<value){
        thershold = value;
        filt = thershold - _touch_threshold;
    }
}
bool TouchBtn::pressed(){
    if(thershold>0){
        return read()>thershold  && has_changed();
    }
    return read()<thershold  && has_changed();
}
uint16_t TouchBtn::read(){
    uint16_t up = 0;
    uint16_t down = 0;
    uint8_t nloop = 100;
    ADMUX = _channel;
    for (uint8_t count = 0; count < nloop; ++count) { 
        // settle, charge
        settle_delay();
        set(*_tx_port, _tx_pin);
        // initiate conversion
        ADCSRA |= (1 << ADSC);
        // wait for completion
        while (ADCSRA & (1 << ADSC));
        // save result
        up += ADC;
        // settle, discharge
        settle_delay();
        clear(*_tx_port, _tx_pin);
        // initiate conversion
        ADCSRA |= (1 << ADSC);
        // wait for completion
        while (ADCSRA & (1 << ADSC));
        // save result
        down += ADC;
    }
    uint16_t value = (up - down)/nloop;
    value*=10;
    filt = (1-eps)*filt + eps*value;
    thershold = filt + _touch_threshold;

    if( !(millis()-_last_press >= _debounce_ms )) return value;
    
    if(thershold >0){
        if((value > thershold) != _state){
            _state = !_state;
            _has_changed = true;
            _last_press = millis();
        }
    }
    else{
        if((value < thershold) != _state){
            _state = !_state;
            _has_changed = true;
            _last_press = millis();
        }
    }

    return value;
}
bool TouchBtn::has_changed(){
    if(_has_changed){
        _has_changed = false;
        return true;
    }
    return false;
}