#include "display.h"
#include <avr/io.h>
#include <Arduino.h>
#include "bitManipulation.h"

volatile uint8_t displayDigits[5] = {10,1,1,1,1};
volatile uint8_t currentDigit=0;
//Display
void Multiplex_init(){
    set(DDRD,PORTD_MULTIPLEX_MASK);
    set(DDRB,PORTB_MULTIPLEX_MASK);

    set(DIGIT012_dir,DIGIT0|DIGIT1|DIGIT2);
    set(DIGIT34_dir,DIGIT3|DIGIT4);
    set(DIGIT012_port,DIGIT0|DIGIT1|DIGIT2);
    set(DIGIT34_port,DIGIT3|DIGIT4);
    cli();
    TCCR2A = (0<<COM1A1) | (0<<WGM21) | (0<<WGM20); // set PWM fast mode on pin 11
    TCCR2B = (1<<CS22) | (0<<CS21) | (0<<CS20); //set prescaler to 1/128 for multiplaxing
    OCR2A = 255;
    TIMSK2 |= (1<<TOIE2); // enable overflow interrupt on timer 2 for multiplex
    sei();
}
ISR(TIMER2_OVF_vect){
    _displayNum(displayDigits[currentDigit]);

    //source from all digits turn they off
    set(DIGIT012_port,DIGIT0|DIGIT1|DIGIT2); 
    set(DIGIT34_port,DIGIT3|DIGIT4);

    switch (currentDigit) // sink from the selected digit
    {
        case 0:clear(DIGIT012_port,DIGIT0); break;
        case 1:clear(DIGIT012_port,DIGIT1); break;
        case 2:clear(DIGIT012_port,DIGIT2); break;
        case 3:clear(DIGIT34_port,DIGIT3); break;
        case 4:clear(DIGIT34_port,DIGIT4); break;
    default: break;
    }
    ++currentDigit;
    currentDigit %= 5;
}
void _displayNum(uint8_t num){
    clear(PORTD,PORTD_MULTIPLEX_MASK);
    clear(PORTB,PORTB_MULTIPLEX_MASK);

    uint8_t digit = digits[num];
    uint8_t portD = ((PORTD_MULTIPLEX_MASK & ~(1<<PD7) ) & (digit<<2)) | ((1<<PD7)&(digit<<4)) | ((1<<PD1)&(digit>>5));
    PORTD |= PORTD_MULTIPLEX_MASK&portD;
    PORTB |= PORTB_MULTIPLEX_MASK&(digit>>4);
}
void DisplayInt(int value){
  if(value>9999) return;
  uint8_t buffer[4] = {10,10,10,10};
  for (uint8_t i = 0; i < 4; i++)
  {
    buffer[i] = value%10;
    value = value /10;
  }
  //Set the display digits
  displayDigits[1] = buffer[3];
  displayDigits[2] = buffer[2];
  displayDigits[3] = buffer[1];
  displayDigits[4] = buffer[0];
}