#include <Arduino.h>
#include <TouchBtn.h>
#include <Wire.h>

#include "bitManipulation.h"
#include "wireHelpers.h"
#include "display.h"
#include "light.h"


#define PUMP_1 5 //PD5
#define PUMP_2 6 //PD6
#define PUMP_ACTIVE_TIME 92000 //ms -> 92s time to fill the tray 
#define SOIL_MOISTURE_CHECK_INTERVAL 43200000 //test 148s //ms -> 12h

#define WATER_SENSOR_ADDRESS 9
#define SOIL_MOISTURE_SUB_ADDRESS 0
#define TANK_LEVEL_SUB_ADDRESS 2

//Time variables
unsigned long _soil_check_interval = SOIL_MOISTURE_CHECK_INTERVAL;
unsigned long _last_soil_check_time = 0xFFFFFFFF;
unsigned long _pump_activation_time = 0xFFFFFFFF - PUMP_ACTIVE_TIME;
bool isTankEmpty = false;
bool irrigationOn = false;
int pumpCycles = 0;

//function declarations
bool evaluateLightValue(unsigned int);
unsigned int handleSoilSensor();
void handleTankSensor();
void updatePump();

#define TX_BTN_PIN (1<<PC0)
TouchBtn midBtn(&PORTC,&DDRC,TX_BTN_PIN,1);
TouchBtn upBtn(&PORTC,&DDRC,TX_BTN_PIN,6);
TouchBtn lowBtn(&PORTC,&DDRC,TX_BTN_PIN,7);

void setup() {
  //CLKPR – Clock Prescale Register changed to divide clock signal by 1
  CLKPR = (1<<CLKPCE); //10000000
  CLKPR = (0<<CLKPS3)|(0<<CLKPS2)|(0<<CLKPS1)|(0<<CLKPS0); //00000000
  
  // setup pins
  pinMode(LIGHT_PIN, OUTPUT);
  pinMode(PUMP_1,OUTPUT);
  pinMode(PUMP_2,OUTPUT);
  // set up buttons
  #define DEBOUNCE_TIME 10
  upBtn.begin(-45,DEBOUNCE_TIME);
  midBtn.begin(-30,DEBOUNCE_TIME);
  lowBtn.begin(45,DEBOUNCE_TIME);
  //speed up 
  TCCR1B = (TCCR1B & B11111000) | B00000001;

  Multiplex_init();

  uint8_t calibrationCount = 100;   
  Wire.begin(); 
  while (calibrationCount > 0){
    upBtn.calibrate();
    midBtn.calibrate();
    lowBtn.calibrate();
    --calibrationCount;
    _delay_us(100);
  }
  displayDigits[0] = 11; //turn on buttons
}

int value = 1046;
void loop() {
  if(midBtn.pressed()){
    lightOnOverride = !lightOnOverride;
  }
  if(upBtn.pressed()){
    ++value;
    DisplayInt(value);
  }
  else if(lowBtn.pressed()){
    --value;
    DisplayInt(value);
  }
  
  handleLightSensor();
  handleTankSensor();
  unsigned int soil = handleSoilSensor();
  DisplayInt(soil);
  if(millis() - _last_soil_check_time >= _soil_check_interval){
    _last_soil_check_time = millis();
    if(isTankEmpty) return;

    if(soil>=70 && irrigationOn){
      irrigationOn = false;
    }
    if(soil<40 || irrigationOn){
      irrigationOn = true;
      _soil_check_interval = PUMP_ACTIVE_TIME + 1;
      _pump_activation_time = millis();
      ++pumpCycles;
    }
    else{
      _soil_check_interval = SOIL_MOISTURE_CHECK_INTERVAL - (PUMP_ACTIVE_TIME*pumpCycles);
      pumpCycles=0;
    }
  }
  updatePump();
}
void updatePump(){
  if(millis() - _pump_activation_time >= PUMP_ACTIVE_TIME){
    digitalWrite(PUMP_1, LOW);
    digitalWrite(PUMP_2, LOW);
  }
  else{
    digitalWrite(PUMP_1, LOW);
    analogWrite(PUMP_2, 70);
  }
}
void handleTankSensor(){
  Wire.beginTransmission(WATER_SENSOR_ADDRESS);
  Wire.write(TANK_LEVEL_SUB_ADDRESS);
  Wire.endTransmission();
  unsigned int value = requestUINT(WATER_SENSOR_ADDRESS);
  if(value < 34){
    isTankEmpty = true;
    displayDigits[0] = 12; //turn on water signal
  }
  else if(value >= 100){
    isTankEmpty = false;
    displayDigits[0] = 11;
  }
}
unsigned int handleSoilSensor(){
  Wire.beginTransmission(WATER_SENSOR_ADDRESS);
  Wire.write(SOIL_MOISTURE_SUB_ADDRESS);
  Wire.endTransmission();
  unsigned int value = requestUINT(WATER_SENSOR_ADDRESS);
  return value;
}


